"""
Tests for the cycle detection functionality in schema parsing.
"""

import importlib  # For reloading
import os
import unittest
from typing import Any, Dict, cast

import pyopenapi_gen.core.parsing.schema_parser as schema_parser  # Import as schema_parser
from pyopenapi_gen.core.parsing.context import ParsingContext
from pyopenapi_gen.core.parsing.schema_parser import _parse_schema
from pyopenapi_gen.core.utils import NameSanitizer
from pyopenapi_gen.ir import IRSchema


class TestCycleDetection(unittest.TestCase):
    def setUp(self) -> None:
        """Set up test environment."""
        self.context = ParsingContext()

        # Store original environment variables
        self.original_env = {
            "PYOPENAPI_DEBUG_CYCLES": os.environ.get("PYOPENAPI_DEBUG_CYCLES"),
            "PYOPENAPI_MAX_CYCLES": os.environ.get("PYOPENAPI_MAX_CYCLES"),
            "PYOPENAPI_MAX_DEPTH": os.environ.get("PYOPENAPI_MAX_DEPTH"),
        }

        # Set test-specific environment variables
        os.environ["PYOPENAPI_DEBUG_CYCLES"] = "1"
        os.environ["PYOPENAPI_MAX_CYCLES"] = "10"  # This is for schema_parser.MAX_CYCLES
        os.environ["PYOPENAPI_MAX_DEPTH"] = "10"  # This is for schema_parser.ENV_MAX_DEPTH

        # Reload schema_parser to pick up new env var values for its module-level constants
        importlib.reload(schema_parser)

    def tearDown(self) -> None:
        """Clean up test environment."""
        # Restore original environment variables
        for key, value in self.original_env.items():
            if value is not None:
                os.environ[key] = value
            elif key in os.environ:  # If original was None (not set), delete it
                del os.environ[key]

        # Reload schema_parser to reflect original/default environment variables
        importlib.reload(schema_parser)

    def test_self_reference_cycle_detection(self) -> None:
        """Test detection of a schema that directly references itself."""
        schema_name = "SelfRefSchema"
        schema_data: Dict[str, Any] = {
            "type": "object",
            "properties": {"name": {"type": "string"}, "self_ref": {"$ref": f"#/components/schemas/{schema_name}"}},
        }
        self.context.raw_spec_schemas = {schema_name: schema_data}
        result = _parse_schema(schema_name, schema_data, self.context, allow_self_reference=False)

        # Verify cycle detection
        self.assertTrue(result._is_circular_ref, "Schema should be marked as circular")
        self.assertTrue(result._from_unresolved_ref, "Schema should be marked as unresolved")
        self.assertIsNotNone(result._circular_ref_path)
        assert result._circular_ref_path is not None
        self.assertIn(schema_name, result._circular_ref_path, "Cycle path should include original schema name")
        self.assertEqual(result.name, NameSanitizer.sanitize_class_name(schema_name), "Schema name should be sanitized")
        self.assertEqual(result.properties, {}, "Circular placeholder should have no properties")

    def test_mutual_reference_cycle_detection(self) -> None:
        """Test detection of mutual references between schemas."""
        schema_a_name = "SchemaA"
        schema_b_name = "SchemaB"
        schema_a: Dict[str, Any] = {
            "type": "object",
            "properties": {"b": {"$ref": f"#/components/schemas/{schema_b_name}"}},
        }
        schema_b: Dict[str, Any] = {
            "type": "object",
            "properties": {"a": {"$ref": f"#/components/schemas/{schema_a_name}"}},
        }

        self.context.raw_spec_schemas = {schema_a_name: schema_a, schema_b_name: schema_b}

        # Parse schema A
        result_a = _parse_schema(schema_a_name, schema_a, self.context, allow_self_reference=False)

        # Verify cycle detection - context should detect the cycle overall
        self.assertTrue(self.context.cycle_detected, "Cycle should be detected overall")
        # SchemaA itself is not marked as circular because it doesn't directly self-reference
        # The cycle is detected when SchemaB references SchemaA (which is already being parsed)
        self.assertFalse(result_a._is_circular_ref, "SchemaA should not be marked as circular (no direct self-ref)")
        self.assertFalse(result_a._from_unresolved_ref, "SchemaA should not be marked as unresolved")
        self.assertEqual(
            result_a.name, NameSanitizer.sanitize_class_name(schema_a_name), "Schema name should be sanitized"
        )

    def test_composition_cycle_detection(self) -> None:
        """Test detection of cycles in schema composition (allOf, anyOf, oneOf)."""
        schema_a_name = "SchemaA"
        schema_b_name = "SchemaB"
        schema_a: Dict[str, Any] = {
            "type": "object",
            "allOf": [
                {"type": "object", "properties": {"name": {"type": "string"}}},
                {"$ref": f"#/components/schemas/{schema_b_name}"},
            ],
        }
        schema_b: Dict[str, Any] = {
            "type": "object",
            "anyOf": [
                {"type": "object", "properties": {"id": {"type": "integer"}}},
                {"$ref": f"#/components/schemas/{schema_a_name}"},
            ],
        }

        self.context.raw_spec_schemas = {schema_a_name: schema_a, schema_b_name: schema_b}

        # Parse schema A
        result_a = _parse_schema(schema_a_name, schema_a, self.context, allow_self_reference=False)

        # Verify cycle detection
        self.assertTrue(self.context.cycle_detected, "Cycle should be detected in composition")
        # SchemaA itself is not marked as circular because it doesn't directly self-reference
        # The cycle is detected when SchemaB references SchemaA (which is already being parsed)
        self.assertFalse(result_a._is_circular_ref, "SchemaA should not be marked as circular (no direct self-ref)")
        self.assertFalse(result_a._from_unresolved_ref, "SchemaA should not be marked as unresolved")
        self.assertEqual(
            result_a.name, NameSanitizer.sanitize_class_name(schema_a_name), "Schema name should be sanitized"
        )

    def test_nested_property_cycle_detection(self) -> None:
        """Test detection of cycles in nested properties."""
        schema_name = "NestedSchema"
        schema_data: Dict[str, Any] = {
            "type": "object",
            "properties": {
                "nested": {
                    "type": "object",
                    "properties": {
                        "deep": {
                            "type": "object",
                            "properties": {"ref": {"$ref": f"#/components/schemas/{schema_name}"}},
                        }
                    },
                }
            },
        }

        self.context.raw_spec_schemas = {schema_name: schema_data}
        result = _parse_schema(schema_name, schema_data, self.context, allow_self_reference=False)

        # Verify cycle detection
        self.assertTrue(result._is_circular_ref, "Schema should be marked as circular")
        self.assertTrue(result._from_unresolved_ref, "Schema should be marked as unresolved")
        self.assertIsNotNone(result._circular_ref_path)
        assert result._circular_ref_path is not None
        self.assertIn(schema_name, result._circular_ref_path, "Cycle path should include original schema name")
        self.assertEqual(result.name, NameSanitizer.sanitize_class_name(schema_name), "Schema name should be sanitized")

    def test_max_recursion_depth(self) -> None:
        """Test handling of maximum recursion depth."""
        schema_name = "DeepSchema"
        # Create a deeply nested schema
        current_schema: Dict[str, Any] = {"type": "string"}
        for _ in range(15):  # Exceed max depth of 10 (set by ENV_MAX_DEPTH in setUp)
            current_schema = {"type": "array", "items": current_schema}

        self.context.raw_spec_schemas = {schema_name: current_schema}
        result = _parse_schema(schema_name, current_schema, self.context, allow_self_reference=False)

        # Verify max depth handling
        # The 'result' (DeepSchema IR) itself may not be the circular placeholder if depth is hit in an anonymous item.
        # However, the context should flag that a cycle (due to max depth) was detected.
        self.assertTrue(self.context.cycle_detected, "Context should flag cycle detected due to max depth")
        self.assertEqual(
            result.name,
            NameSanitizer.sanitize_class_name(schema_name),
            "Schema name should be preserved for the main schema",
        )
        # result._is_circular_ref might be False if the placeholder is for an anonymous item.
        # self.assertTrue(result._is_circular_ref, "Schema should be marked as circular")
        # self.assertTrue(result._from_unresolved_ref, "Schema should be marked as unresolved")
        # self.assertIsNotNone(result._circular_ref_path)
        # assert result._circular_ref_path is not None
        # self.assertIn(NameSanitizer.sanitize_class_name(schema_name), result._circular_ref_path.split(" -> ")[0])
        # self.assertIn("MAX_DEPTH_EXCEEDED", result._circular_ref_path, "Cycle path should indicate max depth")

    def test_environment_variable_effects(self) -> None:
        """
        Test that environment variables like DEBUG_CYCLES are processed without error.
        ENV_MAX_DEPTH is tested more specifically in test_max_recursion_depth and
        test_invalid_env_vars_fallback_to_defaults. MAX_CYCLES is currently not used by the core parsing
        logic for cycle limits.
        """
        # PYOPENAPI_DEBUG_CYCLES is set to "1" in setUp, and schema_parser is reloaded.
        # This test primarily ensures that this setup doesn't cause errors and parsing proceeds.
        # A more thorough test would mock the logger to check for debug messages.

        # schema_parser.DEBUG_CYCLES should be True due to setUp
        # schema_parser.MAX_CYCLES should be 10 due to setUp
        # schema_parser.ENV_MAX_DEPTH should be 10 due to setUp
        # This test primarily ensures the module loads with defaults correctly.

        # schema_parser.DEBUG_CYCLES should be True due to setUp

        context = ParsingContext()
        schema_name = "TestSchemaEnvEffect"
        schema_data = {
            "type": "object",
            "properties": {"self_ref": {"$ref": f"#/components/schemas/{schema_name}"}},
        }
        context.raw_spec_schemas = {schema_name: schema_data}
        result = _parse_schema(schema_name, schema_data, context)

        # Basic check that parsing completed and detected the cycle
        self.assertTrue(result._is_circular_ref, "Schema should be marked as circular")
        self.assertTrue(result._from_unresolved_ref, "Schema should be marked as unresolved")
        self.assertIsNotNone(result._circular_ref_path)
        assert result._circular_ref_path is not None
        self.assertIn(schema_name, result._circular_ref_path, "Cycle path should include original schema name")
        self.assertEqual(result.name, NameSanitizer.sanitize_class_name(schema_name), "Schema name should be sanitized")

    def test_array_items_cycle_detection(self) -> None:
        """Test detection of a cycle where array items refer back to the parent schema."""
        schema_name = "ArrayCycleSchema"
        schema_data: Dict[str, Any] = {
            "type": "object",
            "properties": {
                "name": {"type": "string"},
                "children": {
                    "type": "array",
                    "items": {"$ref": f"#/components/schemas/{schema_name}"},
                },
            },
        }
        self.context.raw_spec_schemas = {schema_name: schema_data}
        result = _parse_schema(schema_name, schema_data, self.context, allow_self_reference=False)

        # Verify cycle detection
        self.assertTrue(result._is_circular_ref, "Schema should be marked as circular due to array items cycle")
        self.assertTrue(result._from_unresolved_ref, "Schema should be marked as unresolved")
        self.assertIsNotNone(result._circular_ref_path, "Circular ref path should be set")
        assert result._circular_ref_path is not None
        self.assertIn(schema_name, result._circular_ref_path, "Cycle path should include original schema name")
        self.assertEqual(result.name, NameSanitizer.sanitize_class_name(schema_name), "Schema name should be sanitized")
        # The properties might be empty or contain 'name' depending on when cycle is detected
        # For this specific case, where the cycle is in 'items', the top-level properties might parse before cycle is
        # fully established.
        #
        # However, the IRSchema for 'children'->'items' should be the circular placeholder.
        # Let's check if the context has the cycle detected flag.
        self.assertTrue(self.context.cycle_detected, "Context should flag that a cycle was detected")

    def test_three_schema_cycle_detection(self) -> None:
        """Test detection of a cycle involving three schemas (A -> B -> C -> A)."""
        schema_a_name = "SchemaA_Triple"
        schema_b_name = "SchemaB_Triple"
        schema_c_name = "SchemaC_Triple"

        schema_a: Dict[str, Any] = {
            "type": "object",
            "properties": {"b_ref": {"$ref": f"#/components/schemas/{schema_b_name}"}},
        }
        schema_b: Dict[str, Any] = {
            "type": "object",
            "properties": {"c_ref": {"$ref": f"#/components/schemas/{schema_c_name}"}},
        }
        schema_c: Dict[str, Any] = {
            "type": "object",
            "properties": {"a_ref": {"$ref": f"#/components/schemas/{schema_a_name}"}},
        }

        self.context.raw_spec_schemas = {
            schema_a_name: schema_a,
            schema_b_name: schema_b,
            schema_c_name: schema_c,
        }

        # Parse schema A, which should trigger the cycle detection through B and C
        result_a = _parse_schema(schema_a_name, schema_a, self.context, allow_self_reference=False)

        # Verify cycle detection - context should detect the cycle overall
        self.assertTrue(self.context.cycle_detected, "Cycle should be detected in three-way cycle")
        # SchemaA_Triple itself is not marked as circular because it doesn't directly self-reference
        # The cycle is detected when one of the schemas in the chain references back to an already-parsing schema
        self.assertFalse(
            result_a._is_circular_ref, f"{schema_a_name} should not be marked as circular (no direct self-ref)"
        )
        self.assertFalse(result_a._from_unresolved_ref, f"{schema_a_name} should not be marked as unresolved")
        self.assertEqual(
            result_a.name, NameSanitizer.sanitize_class_name(schema_a_name), "Schema name should be sanitized"
        )

        # Also check that SchemaB_Triple and SchemaC_Triple are in parsed_schemas.
        # Verify that the context detected the cycle overall
        self.assertTrue(self.context.cycle_detected, "Context should flag that a cycle was detected")
        # Note: Not all schemas in the cycle may be in parsed_schemas if cycle was detected early

        # Ensure that parsing any schema in the cycle detects the cycle overall
        # Clear context for Schema B check (except raw_spec_schemas)
        self.context.clear_cycle_state()
        self.context.parsed_schemas.clear()
        result_b_direct = _parse_schema(schema_b_name, schema_b, self.context, allow_self_reference=False)
        self.assertTrue(self.context.cycle_detected, "Cycle should be detected when parsing B directly")
        # SchemaB itself won't be marked as circular, but some schema in the chain will detect the cycle
        self.assertEqual(result_b_direct.name, NameSanitizer.sanitize_class_name(schema_b_name))

        # Clear context for Schema C check
        self.context.clear_cycle_state()
        self.context.parsed_schemas.clear()
        result_c_direct = _parse_schema(schema_c_name, schema_c, self.context, allow_self_reference=False)
        self.assertTrue(self.context.cycle_detected, "Cycle should be detected when parsing C directly")
        # SchemaC itself won't be marked as circular, but some schema in the chain will detect the cycle
        self.assertEqual(result_c_direct.name, NameSanitizer.sanitize_class_name(schema_c_name))

    def test_invalid_env_vars_fallback_to_defaults(self) -> None:
        """Test that parser falls back to defaults if env vars for limits are invalid."""
        original_max_cycles = os.environ.get("PYOPENAPI_MAX_CYCLES")
        original_max_depth = os.environ.get("PYOPENAPI_MAX_DEPTH")

        try:
            os.environ["PYOPENAPI_MAX_CYCLES"] = "not-an-integer"
            os.environ["PYOPENAPI_MAX_DEPTH"] = "another-bad-value"

            # Reload schema_parser to pick up the modified (invalid) env vars
            # and exercise the try-except blocks for default fallbacks.
            importlib.reload(schema_parser)

            # Check if the module-level constants fell back to defaults
            self.assertEqual(schema_parser.MAX_CYCLES, 0, "MAX_CYCLES should default to 0 on invalid env var")
            self.assertEqual(schema_parser.ENV_MAX_DEPTH, 150, "ENV_MAX_DEPTH should default to 150 on invalid env var")

            # Optional: Perform a minimal parse to ensure no crash and defaults are used by context if applicable
            # This part depends on how ParsingContext gets its max_depth. If it's from ENV_MAX_DEPTH at instantiation,
            # this is good.
            # If schema_parser.ENV_MAX_DEPTH is passed to ParsingContext or _parse_schema, then checking the module var
            # is sufficient. Based on current code, ParsingContext does not directly use ENV_MAX_DEPTH from
            # schema_parser module after import. The _parse_schema function *does* use context.max_depth, which is
            # initialized in ParsingContext. The test for max_recursion_depth already covers the behavior when
            # context.max_depth is hit. This test primarily ensures the module loads with defaults correctly.

        finally:
            # Restore original environment variables
            if original_max_cycles is not None:
                os.environ["PYOPENAPI_MAX_CYCLES"] = original_max_cycles
            elif "PYOPENAPI_MAX_CYCLES" in os.environ:
                del os.environ["PYOPENAPI_MAX_CYCLES"]

            if original_max_depth is not None:
                os.environ["PYOPENAPI_MAX_DEPTH"] = original_max_depth
            elif "PYOPENAPI_MAX_DEPTH" in os.environ:
                del os.environ["PYOPENAPI_MAX_DEPTH"]

            # Reload schema_parser again to restore its state based on original/default env vars for other tests
            importlib.reload(schema_parser)

    def test_simple_self_reference(self) -> None:
        schema_name = "SelfReferencingSchema"
        schema_data = {"properties": {"next": {"$ref": f"#/components/schemas/{schema_name}"}}}
        self.context.raw_spec_schemas[schema_name] = schema_data
        result = _parse_schema(schema_name, schema_data, self.context, allow_self_reference=False)
        self.assertTrue(result._is_circular_ref, "Schema should be marked as a circular reference")

    def test_indirect_cycle(self) -> None:
        schema_a_name = "SchemaA"
        schema_b_name = "SchemaB"
        schema_a = {"properties": {"b": {"$ref": f"#/components/schemas/{schema_b_name}"}}}
        schema_b = {"properties": {"a": {"$ref": f"#/components/schemas/{schema_a_name}"}}}
        self.context.raw_spec_schemas.update({schema_a_name: schema_a, schema_b_name: schema_b})
        result_a = _parse_schema(schema_a_name, schema_a, self.context, allow_self_reference=False)
        self.assertFalse(result_a._is_circular_ref, "SchemaA itself is not the direct circular ref point initially")

    def test_cycle_via_allof(self) -> None:
        schema_a_name = "SchemaAllOfA"
        schema_b_name = "SchemaAllOfB"
        schema_a = {"allOf": [{"$ref": f"#/components/schemas/{schema_b_name}"}]}
        schema_b = {"properties": {"a": {"$ref": f"#/components/schemas/{schema_a_name}"}}}
        self.context.raw_spec_schemas.update({schema_a_name: schema_a, schema_b_name: schema_b})
        result_a = _parse_schema(schema_a_name, schema_a, self.context, allow_self_reference=False)
        self.assertFalse(result_a._is_circular_ref, "SchemaAllOfA might not be the direct circular ref point.")

    def test_no_cycle(self) -> None:
        schema_name = "NonCyclicSchema"
        schema_data = {"properties": {"value": {"type": "string"}}}
        self.context.raw_spec_schemas[schema_name] = schema_data
        result = _parse_schema(schema_name, schema_data, self.context, allow_self_reference=False)
        self.assertFalse(result._is_circular_ref, "Schema should not be marked as circular")

    def test_max_recursion_depth_in_properties(self) -> None:
        max_depth = 5
        original_max_depth = os.environ.get("PYOPENAPI_MAX_DEPTH")
        os.environ["PYOPENAPI_MAX_DEPTH"] = str(max_depth)
        importlib.reload(schema_parser)

        schema_name = "DeepPropertySchema"
        current_schema_dict_level: Dict[str, Any] = {}
        # Build the initial root of the schema for DeepPropertySchema
        self.context.raw_spec_schemas[schema_name] = current_schema_dict_level

        temp_schema_builder = current_schema_dict_level
        # Create nesting up to max_depth + 2 levels (0 to max_depth + 1)
        # The actual schema that hits the limit will be at depth max_depth + 1
        for i in range(max_depth + 2):
            prop_val_node = {"type": "object"}  # Each property is an object, potentially with its own properties
            temp_schema_builder["properties"] = {f"prop{i}": prop_val_node}
            if i < max_depth + 1:  # Don't try to create properties for the one that will exceed depth
                temp_schema_builder = prop_val_node  # Move deeper for the next iteration
            # else: temp_schema_builder for prop{max_depth+1} is the one that will be the placeholder

        result = _parse_schema(
            schema_name, self.context.raw_spec_schemas[schema_name], self.context, allow_self_reference=False
        )

        self.assertFalse(
            result._max_depth_exceeded_marker, "Top-level schema itself should not be marked for max depth"
        )

        # Navigate to the deeply nested property that should have the marker
        # Depth 0: result (DeepPropertySchema)
        # Depth 1: prop0 refers to DeepPropertySchemaProp0
        # Depth 2: prop1 refers to DeepPropertySchemaProp0Prop1
        # ...
        # Depth N: prop(N-1) refers to DeepPropertySchema...Prop(N-1)
        # Max depth is 5. The schema at depth 6 (recursion_depth=6) is where ENV_MAX_DEPTH (5) is exceeded.
        # This is the schema for prop5, which would be named DeepPropertySchemaProp0Prop1Prop2Prop3Prop4Prop5
        # The actual placeholder is the result of parsing the *node* for prop{max_depth} = prop4
        # The iteration goes from i=0 to max_depth+1 (0 to 6 for max_depth=5)
        # prop0, prop1, prop2, prop3, prop4 will be parsed normally.
        # When parsing prop4 (i=4), its properties will try to parse prop5 (i=5).
        # The promoted name for prop_schema_node of prop4 will be DeepPropertySchemaProp0Prop1Prop2Prop3Prop4.
        # Inside this, parsing its property "prop5" will make a promoted name like
        # DeepPropertySchemaProp0Prop1Prop2Prop3Prop4Prop5
        # This is at depth 6. This is where the marker should be.

        current_ir_level = result
        # Loop up to prop{max_depth-2}, whose _refers_to_schema will be the one for prop{max_depth-1}.
        # The schema for prop{max_depth-1} (e.g. ...Prop4 for max_depth=5) is the one that gets marked.
        for i in range(max_depth):  # i goes from 0 to max_depth-1
            self.assertIn(f"prop{i}", current_ir_level.properties)
            prop_ir = current_ir_level.properties[f"prop{i}"]
            self.assertIsNotNone(prop_ir._refers_to_schema, f"prop{i} should refer to a schema")
            referred_schema = cast(IRSchema, prop_ir._refers_to_schema)
            if i < max_depth - 1:  # Schemas for prop0 through prop{max_depth-2} should not be marked
                self.assertFalse(
                    referred_schema._max_depth_exceeded_marker,
                    f"Schema for prop{i} ({referred_schema.name}) should not be marked yet",
                )
            elif i == max_depth - 1:  # Schema for prop{max_depth-1} (e.g. ...Prop4) IS the one that should be marked.
                self.assertTrue(
                    referred_schema._max_depth_exceeded_marker,
                    f"Schema for prop{i} ({referred_schema.name}) should be marked for max depth",
                )
                expected_placeholder_name_segment = NameSanitizer.sanitize_class_name(f"prop{i}")
                self.assertIsNotNone(referred_schema.name, "Placeholder for depth exceeded schema should have a name.")
                self.assertTrue(
                    cast(str, referred_schema.name).endswith(expected_placeholder_name_segment),
                    f"Placeholder name {referred_schema.name} should end with {expected_placeholder_name_segment}",
                )
            current_ir_level = referred_schema

        # The old assertions for navigating one more level are removed as the marker is on prop{max_depth-1}'s schema
        # itself.
        # self.assertIn(f"prop{max_depth}", current_ir_level.properties)
        # final_prop_ir = current_ir_level.properties[f"prop{max_depth}"]
        # _temp_referred_schema = final_prop_ir._refers_to_schema
        # self.assertIsNotNone(_temp_referred_schema, f"prop{max_depth} should refer to a schema")
        # depth_exceeded_schema = cast(IRSchema, _temp_referred_schema)
        # self.assertTrue(depth_exceeded_schema._max_depth_exceeded_marker, f"Schema for prop{max_depth} content should
        # be marked for max depth")

        if original_max_depth is None:
            del os.environ["PYOPENAPI_MAX_DEPTH"]
        else:
            os.environ["PYOPENAPI_MAX_DEPTH"] = original_max_depth
        importlib.reload(schema_parser)

    def test_max_recursion_depth_in_allof(self) -> None:
        max_depth = 3
        original_max_depth = os.environ.get("PYOPENAPI_MAX_DEPTH")
        os.environ["PYOPENAPI_MAX_DEPTH"] = str(max_depth)
        importlib.reload(schema_parser)

        schema_name = "DeepAllOfSchema"
        schemas_to_add: Dict[str, Any] = {}
        current_level_schema_name = schema_name
        # Create schemas A -> B -> C -> D ... via allOf
        # Max depth 3. So, schema_0 allOf schema_1, schema_1 allOf schema_2, schema_2 allOf schema_3.
        # Parsing schema_3 will be depth 4, exceeding limit 3.
        for i in range(max_depth + 2):  # 0, 1, 2, 3, 4 for max_depth=3. Limit hit at i=max_depth (schema_3)
            next_level_schema_name = f"{schema_name}_{i + 1}"
            if i < max_depth + 1:  # For schema_0, schema_1, schema_2, schema_3
                schema_node = {"allOf": [{"$ref": f"#/components/schemas/{next_level_schema_name}"}]}
            else:  # For schema_4 (which won't be reached if depth limit works for schema_3's ref)
                schema_node = {"type": "string"}  # Terminal node if we got this far
            schemas_to_add[current_level_schema_name] = schema_node
            if i == max_depth:  # schema_3 is the one that will be a placeholder
                schemas_to_add[next_level_schema_name] = {
                    "type": "object",
                    "description": "This is the node that should be a placeholder due to depth limit.",
                }  # Provide node for schema_3 to be parsed
            current_level_schema_name = next_level_schema_name

        # Add a terminal node for the deepest reference if not already added (e.g. schema_4 if max_depth=3)
        # This ensures the ref target for schema_3 exists, even if it's schema_4 that would be depth limited.
        # The critical point is schema_3 trying to parse its allOf pointing to schema_4.
        # No, the critical point is when _parse_schema is called *for* schema_3.
        # The setup ensures schema_name (DeepAllOfSchema_0) has schema_1 in its allOf,
        # schema_1 has schema_2, schema_2 has schema_3.
        # Parsing schema_0 (depth 1)
        #  -> calls _parse_schema for schema_1 (depth 2)
        #     -> calls _parse_schema for schema_2 (depth 3)
        #        -> calls _parse_schema for schema_3 (depth 4) -> THIS is where ENV_MAX_DEPTH (3) is exceeded.
        # So, the IRSchema registered for "DeepAllOfSchema_2" should be the placeholder when ENV_MAX_DEPTH is 3.

        context = ParsingContext(raw_spec_schemas=schemas_to_add, raw_spec_components={})
        result = _parse_schema(schema_name, schemas_to_add[schema_name], context, allow_self_reference=False)

        self.assertFalse(result._max_depth_exceeded_marker, "Top-level schema itself should not be marked")

        # The schema that should be marked is DeepAllOfSchema_2 (when max_depth is 3)
        depth_limited_schema_name = f"{schema_name}_{max_depth - 1}"  # e.g. DeepAllOfSchema_2 if max_depth=3
        self.assertIn(
            depth_limited_schema_name,
            context.parsed_schemas,
            f"{depth_limited_schema_name} should be in parsed_schemas as a placeholder",
        )

        _temp_schema_val = context.parsed_schemas[depth_limited_schema_name]
        self.assertIsNotNone(_temp_schema_val, f"{depth_limited_schema_name} should not be None in parsed_schemas")
        depth_exceeded_schema_actual_ir = _temp_schema_val

        self.assertTrue(
            depth_exceeded_schema_actual_ir._max_depth_exceeded_marker,
            f"{depth_limited_schema_name} ({depth_exceeded_schema_actual_ir.name}) "
            f"should be marked for max depth exceeded",
        )

        _temp_name_val = depth_exceeded_schema_actual_ir.name
        self.assertIsNotNone(_temp_name_val, f"{depth_limited_schema_name} placeholder should have a name.")
        schema_actual_name = cast(str, _temp_name_val)
        # The name stored is the sanitized version of the original name used when _handle_max_depth_exceeded was called.
        self.assertEqual(
            schema_actual_name,
            NameSanitizer.sanitize_class_name(depth_limited_schema_name),
            f"Placeholder name {schema_actual_name} should match sanitized {depth_limited_schema_name}",
        )

        # Also, check propagation through the allOf list of the top schema
        # result (DAS_0) -> allOf[0] (DAS_1_IR) -> allOf[0] (DAS_2_IR - placeholder)
        self.assertIsNotNone(result.all_of)
        assert result.all_of is not None
        self.assertTrue(len(result.all_of) > 0)
        das1_ir = result.all_of[0]
        self.assertIsNotNone(
            das1_ir.all_of, "das1_ir.all_of should not be None after previous checks on result.all_of structure"
        )
        assert das1_ir.all_of is not None
        self.assertTrue(len(das1_ir.all_of) > 0)
        das2_ir_placeholder = das1_ir.all_of[0]
        self.assertTrue(
            das2_ir_placeholder._max_depth_exceeded_marker, "DAS_2 placeholder in allOf chain should be marked."
        )
        self.assertEqual(das2_ir_placeholder.name, NameSanitizer.sanitize_class_name(depth_limited_schema_name))

        if original_max_depth is None:
            del os.environ["PYOPENAPI_MAX_DEPTH"]
        else:
            os.environ["PYOPENAPI_MAX_DEPTH"] = original_max_depth
        importlib.reload(schema_parser)

    def test_complex_cycle_with_multiple_refs(self) -> None:
        schema_name = "ComplexCycleStart"
        schema_data = {
            "properties": {
                "ref1": {"$ref": "#/components/schemas/RefSchema1"},
                "ref2": {"$ref": "#/components/schemas/RefSchema2"},
            }
        }
        ref_schema1 = {"properties": {"next": {"$ref": "#/components/schemas/RefSchema3"}}}
        ref_schema2 = {"properties": {"other": {"type": "string"}}}  # Does not cycle
        ref_schema3 = {"properties": {"back": {"$ref": f"#/components/schemas/{schema_name}"}}}  # Cycles back

        self.context.raw_spec_schemas.update(
            {
                schema_name: schema_data,
                "RefSchema1": ref_schema1,
                "RefSchema2": ref_schema2,
                "RefSchema3": ref_schema3,
            }
        )
        result = _parse_schema(schema_name, schema_data, self.context, allow_self_reference=False)
        self.assertFalse(result._is_circular_ref, "Initial schema may not be marked if cycle is indirect")

    def test_cycle_in_array_items(self) -> None:
        schema_a_name = "ArrayCycleA"
        schema_b_name = "ArrayCycleB"
        schema_a = {"type": "array", "items": {"$ref": f"#/components/schemas/{schema_b_name}"}}
        schema_b = {"properties": {"a_again": {"$ref": f"#/components/schemas/{schema_a_name}"}}}
        self.context.raw_spec_schemas.update({schema_a_name: schema_a, schema_b_name: schema_b})

        result_a = _parse_schema(schema_a_name, schema_a, self.context, allow_self_reference=False)
        self.assertFalse(result_a._is_circular_ref)  # Schema A itself is not circular initially
        self.assertIsNotNone(result_a.items)
        if result_a.items and result_a.items._refers_to_schema:
            schema_b_ir = result_a.items._refers_to_schema
            self.assertEqual(schema_b_ir.name, "ArrayCycleB")
            self.assertIsNotNone(schema_b_ir.properties)
            if schema_b_ir.properties:
                a_again_prop = schema_b_ir.properties.get("a_again")
                self.assertIsNotNone(a_again_prop)
                if a_again_prop and a_again_prop._refers_to_schema:
                    self.assertTrue(a_again_prop._refers_to_schema._is_circular_ref)


if __name__ == "__main__":
    unittest.main()
